#############################################################################################################
# Variables
#############################################################################################################
variable "vpc_id" {}
variable "security_groups" {
  type = "list"
}

variable "subnet_ids" {
  type = "list"
}

variable "cache_name" {}
variable "cache_clusters" {
  default = "1"
}
variable "automatic_failover_enabled" {
  default = "false"
}
variable "engine_version" {
  default = "2.8.24"
}
variable "parameter_group_name" {
  default = "default.redis2.8"
}
variable "instance_type" {
  default = "cache.t2.micro"
}
variable "port" {
  default = "6379"
}

variable "maintenance_window" {
  default = "sun:01:00-sun:02:00"
}
variable "snapshot_window" {
  default = "03:00-04:00"
}
variable "snapshot_retention_limit" {
  default = "1"
}

variable "dns_zone" {}
variable "cloudwatch_alarm_sns_arn" {
  type = "list"
}


############################################################################################################
# Security Group
############################################################################################################
resource "aws_security_group" "redis" {
  vpc_id      = "${var.vpc_id}"
  description = "ElastiCache Redis SG"

  ingress {
    from_port       = "${var.port}"
    to_port         = "${var.port}"
    protocol        = "tcp"
    security_groups = ["${var.security_groups}"]
  }
  tags = {
    Name = "${var.cache_name}"
  }
}

#############################################################################################################
# Subnet Group
#############################################################################################################
resource "aws_elasticache_subnet_group" "default" {
  name        = "${var.cache_name}-redis-subnet-group"
  description = "Private subnets for the ElastiCache Redis instances"
  subnet_ids  = ["${var.subnet_ids}"]
}

############################################################################################################
# Elasticache (Redis) Single
############################################################################################################
resource "aws_elasticache_replication_group" "redis-single" {
  count                         = "${replace(replace(var.cache_clusters, "/^[2-9]$/", "0"), "/^1$/", "1")}"
  replication_group_id          = "${var.cache_name}"
  replication_group_description = "${var.cache_name}"
  engine_version                = "${var.engine_version}"
  node_type                     = "${var.instance_type}"
  number_cache_clusters         = "1"
  port                          = "${var.port}"
  parameter_group_name          = "${var.parameter_group_name}"
  subnet_group_name             = "${aws_elasticache_subnet_group.default.name}"
  security_group_ids            = ["${aws_security_group.redis.id}"]
  maintenance_window            = "${var.maintenance_window}"
  auto_minor_version_upgrade    = true

  tags {
    Name = "${var.cache_name}-redis"
  }
}

############################################################################################################
# Elasticache (Redis) Cluster
############################################################################################################
resource "aws_elasticache_replication_group" "redis-cluster" {
  count                         = "${replace(replace(var.cache_clusters, "/^1$/", "0"), "/^[2-9]$/", "1")}"
  replication_group_id          = "${var.cache_name}"
  replication_group_description = "${var.cache_name}"
  engine_version                = "${var.engine_version}"
  node_type                     = "${var.instance_type}"
  number_cache_clusters         = "${var.cache_clusters}"
  port                          = "${var.port}"
  parameter_group_name          = "${var.parameter_group_name}"
  subnet_group_name             = "${aws_elasticache_subnet_group.default.name}"
  security_group_ids            = ["${aws_security_group.redis.id}"]
  maintenance_window            = "${var.maintenance_window}"
  snapshot_window               = "${var.snapshot_window}"
  snapshot_retention_limit      = "${var.snapshot_retention_limit}"
  automatic_failover_enabled    = "${var.automatic_failover_enabled}"
  auto_minor_version_upgrade    = true

  tags {
    Name = "${var.cache_name}-redis"
  }
}

#############################################################################################################
# Route53 records
#############################################################################################################
resource "aws_route53_record" "redis-single-pri" {
  count   = "${replace(replace(var.cache_clusters, "/^[2-9]$/", "0"), "/^1$/", "1")}"
  zone_id = "${var.dns_zone}"
  name    = "redis.pri.${var.cache_name}.aws"
  type    = "CNAME"
  ttl     = "60"
  records = ["${aws_elasticache_replication_group.redis-single.primary_endpoint_address}"]
}

resource "aws_route53_record" "redis-cluster-pri" {
  count   = "${replace(replace(var.cache_clusters, "/^1$/", "0"), "/^[2-9]$/", "1")}"
  zone_id = "${var.dns_zone}"
  name    = "redis.pri.${var.cache_name}.aws"
  type    = "CNAME"
  ttl     = "60"
  records = ["${aws_elasticache_replication_group.redis-cluster.primary_endpoint_address}"]
}

resource "aws_route53_record" "redis-cluster-cfg" {
  count   = "${replace(replace(var.cache_clusters, "/^1$/", "0"), "/^[2-9]$/", "1")}"
  zone_id = "${var.dns_zone}"
  name    = "redis.cfg.${var.cache_name}.aws"
  type    = "CNAME"
  ttl     = "60"
  records = ["${aws_elasticache_replication_group.redis-cluster.configuration_endpoint_address}"]
}

##############################################################################################################
## Cloudwatch resources
##############################################################################################################
resource "aws_cloudwatch_metric_alarm" "cpu-single" {
  count               = "${replace(replace(var.cache_clusters, "/^[2-9]$/", "0"), "/^1$/", "1")}"
  alarm_name          = "alarmRedisCacheClusterCPUUtilization"
  alarm_description   = "Redis cache cluster CPU utilization"
  alarm_actions       = ["${var.cloudwatch_alarm_sns_arn}"]
  comparison_operator = "GreaterThanThreshold"
  evaluation_periods  = "1"
  metric_name         = "CPUUtilization"
  namespace           = "AWS/ElastiCache"
  period              = "300"
  statistic           = "Average"
  threshold           = "75"
  dimensions {
    CacheClusterId     = "${var.cache_name}-00${count.index + 1}"
  }
}

resource "aws_cloudwatch_metric_alarm" "memory-free-single" {
  count               = "${replace(replace(var.cache_clusters, "/^[2-9]$/", "0"), "/^1$/", "1")}"
  alarm_name          = "alarmRedisCacheClusterFreeableMemory"
  alarm_description   = "Redis cache cluster freeable memory"
  alarm_actions       = ["${var.cloudwatch_alarm_sns_arn}"]
  comparison_operator = "LessThanThreshold"
  evaluation_periods  = "1"
  metric_name         = "FreeableMemory"
  namespace           = "AWS/ElastiCache"
  period              = "60"
  statistic           = "Average"
  # 25MB in bytes
  threshold           = "25000000"
  dimensions {
    CacheClusterId     = "${var.cache_name}-00${count.index + 1}"
  }
}

resource "aws_cloudwatch_metric_alarm" "cpu-cluster" {
  count               = "${replace(replace(var.cache_clusters, "/^1$/", "0"), "/^[2-9]$/", "${var.cache_clusters}")}"
  alarm_name          = "alarmRedisCacheCluster00${count.index + 1}CPUUtilization"
  alarm_description   = "Redis cache cluster 00${count.index + 1} CPU utilization"
  alarm_actions       = ["${var.cloudwatch_alarm_sns_arn}"]
  comparison_operator = "GreaterThanThreshold"
  evaluation_periods  = "1"
  metric_name         = "CPUUtilization"
  namespace           = "AWS/ElastiCache"
  period              = "300"
  statistic           = "Average"
  threshold           = "75"
  dimensions {
    CacheClusterId     = "${var.cache_name}-00${count.index + 1}"
  }
}

resource "aws_cloudwatch_metric_alarm" "memory-free-cluster" {
  count               = "${replace(replace(var.cache_clusters, "/^1$/", "0"), "/^[2-9]$/", "${var.cache_clusters}")}"
  alarm_name          = "alarmRedisCacheCluster00${count.index + 1}FreeableMemory"
  alarm_description   = "Redis cache cluster 00${count.index + 1} freeable memory"
  alarm_actions       = ["${var.cloudwatch_alarm_sns_arn}"]
  comparison_operator = "LessThanThreshold"
  evaluation_periods  = "1"
  metric_name         = "FreeableMemory"
  namespace           = "AWS/ElastiCache"
  period              = "60"
  statistic           = "Average"
  # 25MB in bytes
  threshold           = "25000000"
  dimensions {
    CacheClusterId     = "${var.cache_name}-00${count.index + 1}"
  }
}
